<?php
/**
 * Blackbird ContentManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_ContentManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 * @license		http://www.advancedcontentmanager.com/license/
 */
namespace Blackbird\ContentManager\Block\Adminhtml\ContentList\Widget\Grid\Column\Renderer;

use Blackbird\ContentManager\Api\Data\ContentListInterface as ContentListData;

/**
 * StoreView grid column
 */
class Store extends \Magento\Backend\Block\Widget\Grid\Column\Renderer\Store
{
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\ContentList\CollectionFactory
     */
    protected $_contentListCollectionFactory;
    
    /**
     * @param \Magento\Backend\Block\Context $context
     * @param \Magento\Store\Model\System\Store $systemStore
     * @param \Blackbird\ContentManager\Model\ResourceModel\ContentList\CollectionFactory $contentListCollectionFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Context $context,
        \Magento\Store\Model\System\Store $systemStore,
        \Blackbird\ContentManager\Model\ResourceModel\ContentList\CollectionFactory $contentListCollectionFactory,
        array $data = array()
    ) {
        $this->_contentListCollectionFactory = $contentListCollectionFactory;
        parent::__construct($context, $systemStore, $data);
    }
    
    /**
     * Render row store views
     *
     * @param \Magento\Framework\DataObject $row
     * @return \Magento\Framework\Phrase|string
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function render(\Magento\Framework\DataObject $row)
    {
        $contentList = $this->getContentList($row->getData(ContentListData::ID));
        $origStores = [];
        foreach ($contentList->getStores() as $store) {
            $origStores[] = $store;
        }
        $row->setData($this->getColumn()->getIndex(), $origStores);
        
        return parent::render($row);
    }
    
    /**
     * Retrieve the content list from the collection
     * 
     * @param int $contentListId
     * @return \Blackbird\ContentManager\Model\Content
     */
    protected function getContentList($contentListId)
    {
        $contentList = null;
        $collection = $this->_contentListCollectionFactory->create();
        $collection->addFieldToFilter(ContentListData::ID, $contentListId);
        
        if ($collection->getSize() > 0) {
            $contentList = $collection->getFirstItem();
        }
        
        return $contentList;
    }
}
