<?php
/**
 * Blackbird ContentManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_ContentManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 * @license		http://www.advancedcontentmanager.com/license/
 */
namespace Blackbird\ContentManager\Block\Content\Widget;

use Blackbird\ContentManager\Model\Content;

class View extends \Magento\Catalog\Block\Product\AbstractProduct
    implements \Magento\Framework\DataObject\IdentityInterface,
               \Magento\Widget\Block\BlockInterface
{
    /**
     * @var \Blackbird\ContentManager\Model\Config\Source\ContentType\Layouts
     */
    protected $_contentCollectionFactory;
    
    /**
     * @var \Blackbird\ContentManager\Model\Config\Source\ContentType\Layouts
     */
    protected $_sourceLayouts;
    
    /**
     * @var \Magento\Cms\Model\Template\FilterProvider
     */
    protected $_filterProvider;

    /**
     * @param \Magento\Catalog\Block\Product\Context $context
     * @param \Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory $contentCollectionFactory
     * @param \Blackbird\ContentManager\Model\Config\Source\ContentType\Layouts $sourceLayouts
     * @param \Magento\Cms\Model\Template\FilterProvider $filterProvider
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        \Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory $contentCollectionFactory,
        \Blackbird\ContentManager\Model\Config\Source\ContentType\Layouts $sourceLayouts,
        \Magento\Cms\Model\Template\FilterProvider $filterProvider,
        array $data = []
    ) {
        $this->_contentCollectionFactory = $contentCollectionFactory;
        $this->_sourceLayouts = $sourceLayouts;
        $this->_filterProvider = $filterProvider;
        parent::__construct($context, $data);
    }
    
    /**
     * @return void
     */
    protected function _prepareLayout()
    {
        $content = $this->getContent();
        $contentType = $content->getContentType();
        
        // Applied content layout in cascading
        if ($contentType->getLayout() == 0) {
            // Test applying view-"ID".phtml
            $this->setTemplate('Blackbird_ContentManager::content/view/' . $contentType->getIdentifier() . '/view-' . $content->getId() . '.phtml');
            
            if (!$this->getTemplateFile()) {
                // Test applying the overriding content type view.phtml
                $this->setTemplate('Blackbird_ContentManager::content/view/' . $contentType->getIdentifier() . '/view.phtml');
                
                if (!$this->getTemplateFile()) {
                    // Applying default view.phtml
                    $this->setTemplate('Blackbird_ContentManager::content/view/default/view.phtml');
                }
            }
        } else {
            // Test applying view/layout-ID.phtml
            if ($this->_sourceLayouts->layoutExists($contentType->getLayout())) {
                // Build the layout template dynamically
                $this->addChild(
                    'content_view_layout',
                    '\Blackbird\ContentManager\Block\View\Layout',
                    [
                        'layout_template' => $this->_sourceLayouts->retrieveLayout($contentType->getLayout()),
                        'content_type' => $contentType,
                        'content' => $content,
                    ]
                );
            }
            
            // Applying default view.phtml
            $this->setTemplate('Blackbird_ContentManager::content/view/default/view.phtml');
        }

        return parent::_prepareLayout();
    }
    
    /**
     * Get current content
     * 
     * @return \Blackbird\ContentManager\Model\Content
     */
    public function getContent()
    {
        if (!$this->hasData('content') && $this->hasData('content_id')) {
            $contentCollection = $this->_contentCollectionFactory->create()
                ->addAttributeToFilter(Content::ID, $this->getContentId())
                ->addAttributeToSelect('*');
            
            if ($contentCollection->getSize()) {
                $this->setData('content', $contentCollection->getFirstItem());
            }
        }
        
        return $this->getData('content');
    }
    
    /**
     * Return identifiers for produced content
     *
     * @return array
     */
    public function getIdentities()
    {
        return [
            \Blackbird\ContentManager\Model\ContentType::CACHE_TAG . '_' . $this->getContent()->getContentType()->getId(),
            \Blackbird\ContentManager\Model\Content::CACHE_TAG . '_' . $this->getContent()->getId(),
        ];
    }
    
    /**
     * Get the content in HTML format
     * 
     * @return string
     */
    public function getContentHtml()
    {
        $html = '';
        $content = $this->getContent();
        $contentType = $content->getContentType();
        $fields = $contentType->getCustomFieldCollection();
        
        // Layout template, else default template
        if ($this->getChildBlock('content_view_layout') !== false) {
            $html = $this->getChildHtml('content_view_layout');
        } else {
            $html = $content->render('title');
            
            foreach ($fields as $field) {
                $html .= $content->render($field);
            }
        }
        
        return $html;
    }
    
    /**
     * Get processed value
     * 
     * @param string $value
     * @return string
     */
    public function getProcessedData($value)
    {
        $storeId = $this->_storeManager->getStore()->getId();
        return $this->_filterProvider->getBlockFilter()
                ->setStoreId($storeId)
                ->filter($value);
    }
    
}
