<?php
/**
 * Blackbird ContentManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_ContentManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 * @license		http://www.advancedcontentmanager.com/license/
 */
namespace Blackbird\ContentManager\Block\View\Field;

use Blackbird\ContentManager\Model\Content as ContentModel;

class Content extends \Magento\Catalog\Block\Product\AbstractProduct
{
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\Content\Collection
     */
    protected $_contentCollectionInstance;
    
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory
     */
    protected $_contentCollection;
    
    /**
     * @param \Magento\Catalog\Block\Product\Context $context
     * @param \Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory $contentCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        \Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory $contentCollection,
        array $data = array()
    ) {
        $this->_contentCollection = $contentCollection;
        parent::__construct($context, $data);
    }
    
    /**
     * Return the collection of the contents
     * 
     * @param array $attributes
     * @return \Blackbird\ContentManager\Model\ResourceModel\Content\Collection
     */
    public function getContentCollection(array $attributes)
    {
        $contentIds = array_map('trim', explode(',', $this->getContent()->getData($this->getIdentifier())));
        
        $collection = $this->_getContentCollectionInstance()
                        ->addAttributeToFilter(ContentModel::STATUS, 1)
                        ->addAttributeToSelect(array_merge($attributes, ['title', 'url_key']))
                        ->addAttributeToFilter(ContentModel::ID, ['IN' => $contentIds]);
        
        return $collection;
    }
    
    /**
     * Rerieve the content collection instance
     * 
     * @return \Blackbird\ContentManager\Model\ResourceModel\Content\Collection
     */
    protected function _getContentCollectionInstance()
    {
        if (!$this->_contentCollectionInstance) {
            $this->_contentCollectionInstance = $this->_contentCollection->create();
        }
        
        return $this->_contentCollectionInstance;
    }
    
    /**
     * @return $this
     */
    protected function _prepareLayout()
    {
        $content = $this->getContent();
        $contentType = $content->getContentType();
        $type = $this->getType();
        
        // Test applying content/view/"content type"/field/content/"content type"-"ID".phtml
        $this->setTemplate('Blackbird_ContentManager::content/view/' . $contentType->getIdentifier() . '/view/field/content/' . $type . '-' . $content->getId() . '.phtml');
        
        if (!$this->getTemplateFile()) {
            // Test applying content/view/"content type"/field/content/"content type.phtml
            $this->setTemplate('Blackbird_ContentManager::content/view/' . $contentType->getIdentifier() . '/view/field/content/' . $type . '.phtml');
            
            if (!$this->getTemplateFile()) {
                // Applying default content/view/default/field/content/type.phtml
                $this->setTemplate('Blackbird_ContentManager::content/view/default/view/field/content/' . $type . '.phtml');
            }
        }
        
        return parent::_prepareLayout();
    }
}
